;;; -*- mode: lisp; package: infoman -*-
;;; Last updated: <2005/07/08>

(provide "infoman/draft")

(in-package "infoman")

(defun infoman-insert-password (&optional (len *infoman-password-length*)
										  (str *infoman-password-string*))
	"pX[h}"
	(interactive (when *prefix-args*
					 (list (read-integer "Password Length: ")
						   (read-string "Password String: "
										:default *infoman-password-string*))))
	(let (start pstr)
		(dotimes (var len pstr)
			(setf start (random (length str)))
			(setf pstr (concat (substring str start (1+ start))
							   pstr)))
		(if (stringp pstr)
				(insert pstr))))

(defun infoman-popup-completion ()
	"⊮Xg|bvAbv}"
	(interactive)
	(popup-list *infoman-completion-list*
				#'(lambda (x)
					  (insert x)
					  (refresh-screen))))

(defun infoman-draft-replace-string (str)
	"^uAs&tab;A&ret;A󕶎Xy[Xɒu"
	(if (string-equal str "")
			(string " ")
		(substitute-string (substitute-string str "\t" "&tab;") "\n" "&ret;")))

(defun infoman-save-draft ()
	"ҏWeۑ"
	(interactive)
	(let ((str (buffer-substring (point-min) (point-max)))
		  record
		  pos)
		(delete-buffer (selected-buffer))
		(set-buffer (find-buffer *infoman-data-buffer-name*))
		(delete-region (point)
					   (progn
						   (goto-eol)
						   (point)))
		(dolist (var *infoman-list-value-tmp1*)
			(insert var "\t"))
		(insert (infoman-draft-replace-string str))
		(dolist (var *infoman-list-value-tmp2*)
			(insert "\t" var))
		(setf record
			  (buffer-substring (progn
									(goto-bol)
									(point))
								(progn
									(goto-eol)
									(point))))
		(goto-bol)
		(write-file (concat *infoman-data-file-path* *infoman-data-file-extension*))
		(set-buffer-modified-p nil)
		(setf *infoman-list-value-tmp1* nil *infoman-list-value-tmp2* nil)
		(set-buffer (find-buffer *infoman-view-buffer-name*))
		(setf pos (point))
		(other-window)
		(when (string= (buffer-name (selected-buffer)) *infoman-grep-buffer-name*)
			(let ((lnum (current-line-number)))
				(set-buffer (find-buffer *infoman-list-buffer-name*))
				(goto-line (1- (nth (- (list-length *infoman-grep-record-line*) lnum)
									*infoman-grep-record-line*)))))
		(save-excursion
			(setf buffer-read-only nil)
			(delete-region (progn
							   (goto-bol)
							   (point))
						   (progn
							   (forward-line)
							   (point)))
			(infoman-insert-list record)
			(set-text-attribute (point-min)
								(point-max)
								'list
								:foreground *infoman-list-fgcolor*
								:bold *infoman-list-bold*)
			(write-file *infoman-data-file-path* t)
			(set-buffer-modified-p nil)
			(setf buffer-read-only t))
		(other-window -1)
		(setf *infoman-list-item-value* (split-string record #\TAB))
		(infoman-view-list)
		(goto-char pos)
		(when (find-buffer *infoman-grep-buffer-name*)
			(other-window)
			(set-buffer (find-buffer *infoman-grep-buffer-name*))
			(other-window -1))
		(toggle-ime nil)))

(defun infoman-delete-draft ()
	"ҏWej"
	(interactive)
	(setf *infoman-list-value-tmp1* nil *infoman-list-value-tmp2* nil)
	(delete-buffer (selected-buffer))
	(set-buffer (find-buffer *infoman-view-buffer-name*))
	(toggle-ime nil))

(defun infoman-insert-filepath ()
	"t@Ct@CpX擾"
	(interactive)
	(multiple-value-bind (path bool)
			(filer *infoman-filer-directory* nil "t@CpX擾" nil nil)
		(when bool
			(insert path))))

(defun infoman-view-replace-string (str &optional n)
	"󂯎&tab;&ret;^uAsɒu"
	(let ((newline-string "\n"))
		(when n
			(dotimes (var n)
				(setf newline-string (concat newline-string " "))))
		(substitute-string (substitute-string str "&tab;" "\t") "&ret;" newline-string)))

(defun infoman-show-edit-content (str item-num)
	"ҏWO̓e\"
	(let ((list-len (list-length *infoman-list-item-header*)))
		(insert (infoman-view-replace-string str))
		(selection-whole-buffer)
		(set-buffer-modified-p nil)
		(setf *infoman-list-value-tmp1*
			  (butlast *infoman-list-item-value* (- list-len item-num)))
		(setf *infoman-list-value-tmp2*
			  (last *infoman-list-item-value* (- list-len item-num 1)))))

(defun infoman-draft-mode ()
	"htgobt@쐬"
	(set-buffer (get-buffer-create *infoman-draft-buffer-name*))
	(kill-all-local-variables)
	(setf mode-name "Draft")
	(setf buffer-mode 'infoman-draft-mode)
	(use-keymap *infoman-draft-mode-map*)
	(make-local-variable 'mode-line-format)
	(setf mode-line-format "%M")
	(make-local-variable 'need-not-save)
	(setf need-not-save t)
	(make-local-variable 'auto-save)
	(setf auto-save nil)
	(make-local-variable '*filer-last-file-mask*)
	(setf *filer-last-file-mask* '("*"))
	(set-local-window-flags (selected-buffer) *window-flag-ruler* nil)
	(set-buffer-fold-width t)
	(run-hooks '*infoman-draft-mode-hook*))
