;;; -*- mode: lisp; package: infoman -*-
;;; Last updated: <2005/07/08>

(provide "infoman/folder")

(in-package "infoman")

(defun infoman-window-reset ()
	"infomanEBhEZbg"
	(when (find-buffer *infoman-draft-buffer-name*)
		(other-window)
		(set-buffer (find-buffer *infoman-draft-buffer-name*))
		(infoman-delete-draft)
		(other-window -1))
	(erase-buffer (find-buffer *infoman-view-buffer-name*))
	(when (find-buffer *infoman-grep-buffer-name*)
		(other-window -1)
		(set-buffer (find-buffer *infoman-grep-buffer-name*))
		(delete-buffer *infoman-grep-buffer-name*)
		(set-buffer (find-buffer *infoman-list-buffer-name*))
		(erase-buffer (selected-buffer))
		(other-window))
	(when (find-buffer *infoman-data-buffer-name*)
		(delete-buffer *infoman-data-buffer-name*)
		(setf *infoman-list-item-header* nil *infoman-record-regexp* nil))
	(delete-text-attributes 'current-folder))

(defun infoman-list-replace-string (str)
	"󂯎&tab;&ret;Xy[Xɒu"
	(substitute-string (substitute-string str "&tab;" "  ") "&ret;" "  "))

(defun infoman-insert-list (record)
	"Xg}"
	(let (strlist
		  item
		  n)
		(if (or (string-equal *infoman-list-buffer-name*
							  (buffer-name (selected-buffer)))
				(string-equal *infoman-grep-buffer-name*
							  (buffer-name (selected-buffer))))
				(when (string-match *infoman-record-regexp* record)
					(insert " ")
					(setf strlist (split-string (string-replace-match record *infoman-list-item*) #\TAB))
					(dolist (var *infoman-list-item-width*)
						(when (setf item (pop strlist))
							(if (>= (setf n (- var (infoman-calc-length item))) 0)
									(progn
										(insert (infoman-list-replace-string item))
										(insert " " (1+ n)))
								(insert (abbreviate-display-string (infoman-list-replace-string item) var nil)
										" "))))
					(when (setf item (pop strlist))
						(insert item))
					(insert "\n")))))

(defun infoman-record-regexp ()
	"R[hK\"
	(let ((regexp "\\(.*\\)"))
		(setf *infoman-record-regexp* nil)
		(dotimes (var (1- (list-length *infoman-list-item-header*)))
			(setf regexp (concat regexp "\t\\(.*\\)")))
		(setf *infoman-record-regexp* (compile-regexp regexp))))

(defun infoman-get-list (path &optional bool)
	"Xgǂݍ"
	(let (record)
		(set-buffer (get-buffer-create *infoman-data-buffer-name*))
		(erase-buffer (selected-buffer))
		(insert-file (concat path *infoman-data-file-extension*))
		(goto-line 1)
		(set-buffer-modified-p nil)
		(setf *infoman-list-item-header*
			  (split-string (buffer-substring (point)
											  (progn
												  (goto-eol)
												  (point)))
							#\TAB))
		(infoman-record-regexp)
		(set-buffer (find-buffer *infoman-list-buffer-name*))
		(setf buffer-read-only nil)
		(erase-buffer (selected-buffer))
		(if (and (not bool)
				 (file-exist-p path))
				(insert-file path)
			(progn
				(with-input-from-buffer ((find-buffer *infoman-data-buffer-name*))
					(read-line *standard-input* nil)
					(while (setf record (read-line *standard-input* nil))
						(infoman-insert-list record)))
				(write-file path t)))
		(set-text-attribute (point-min)
							(point-max)
							'list
							:foreground *infoman-list-fgcolor*
							:bold *infoman-list-bold*)
		(goto-line 1)
		(set-buffer-modified-p nil)
		(setf buffer-read-only t)))

(defun infoman-folder-init (file)
	"tH_ݒt@Cǂݍ"
	(let ((l (concat file ".l"))
		  (lc (concat file ".lc")))
		(if (file-exist-p "~/site-lisp/infoman/config-folder.lc")
				(load "~/site-lisp/infoman/config-folder.lc")
			(load "~/site-lisp/infoman/config-folder.l"))
		(if (and (file-exist-p lc)
				 (file-newer-than-file-p lc l))
				(load lc)
			(if (file-exist-p l)
					(load l)))))

(defun infoman-get-folder-data-path ()
	"Lbgs̃tH_f[^pXigqj擾"
	(let ((lnum (current-line-number))
		  fname)
		(if (setf fname (nth (1- lnum) *infoman-folder-list*))
				(setf *infoman-data-file-path* (merge-pathnames fname (concat *infoman-data-folder-path* "/" fname)))
			(setf *infoman-data-file-path* nil))))

(defun infoman-open-folder ()
	"Lbgs̃tH_I[v"
	(interactive)
	(unless (eobp)
		(infoman-window-reset)
		(infoman-get-folder-data-path)
		(when (file-exist-p (concat *infoman-data-file-path* *infoman-data-file-extension*))
			(save-excursion
				(set-text-attribute (progn
										(goto-bol)
										(point))
									(progn
										(forward-char)
										(point))
									'current-folder
									:background *infoman-current-folder-bgcolor*))
			(other-window -1)
			(infoman-folder-init *infoman-data-file-path*)
			(infoman-get-list *infoman-data-file-path*))))

(defun infoman-get-folder-list ()
	"tH_Xg擾"
	(set-buffer (create-new-buffer *infoman-work-buffer-name*))
	(if (file-exist-p *infoman-folder-list-file-path*)
			(progn
				(insert-file *infoman-folder-list-file-path*)
				(setf *infoman-folder-list* (split-string (buffer-substring (point-min)
																			(point-max))
														  #\TAB)))
		(if (file-exist-p *infoman-data-folder-path*)
				(progn
					(dolist (var (directory *infoman-data-folder-path* :directory-only t))
						(insert (string-right-trim "/" var) "\t"))
					(delete-char -1)
					(setf *infoman-folder-list* (split-string (buffer-substring (point-min)
																				(point-max))
															  #\TAB))
					(write-file *infoman-folder-list-file-path* t))
			(create-directory *infoman-data-folder-path*)))
	(delete-buffer (selected-buffer))
	(set-buffer (find-buffer *infoman-folder-buffer-name*)))

(defun infoman-show-folders (&optional (lnum 1))
	"tH_\"
	(when (string-equal *infoman-folder-buffer-name*
						(buffer-name (selected-buffer)))
		(setf buffer-read-only nil)
		(erase-buffer (selected-buffer))
		(unless *infoman-folder-list*
			(infoman-get-folder-list))
		(dolist (var *infoman-folder-list*)
			(insert "  " var "\n"))
		(set-text-attribute (point-min)
							(point-max)
							'folder
							:foreground *infoman-folder-fgcolor*
							:bold *infoman-folder-bold*)
		(goto-line lnum)
		(set-buffer-modified-p nil)
		(setf buffer-read-only t)))

(defun infoman-folder-line-up ()
	"ݍs̃tH_ֈړ"
	(interactive)
	(unless (or (eobp)
				(= (current-line-number) 1))
		(let ((lnum (current-line-number))
			  (list-len (list-length *infoman-folder-list*)))
			(infoman-window-reset)
			(set-buffer (create-new-buffer *infoman-work-buffer-name*))
			(if (= lnum 2)
					(progn
						(insert (nth 1 *infoman-folder-list*) "\t")
						(insert (nth 0 *infoman-folder-list*) "\t")
						(dolist (var (last *infoman-folder-list* (- list-len 2)))
							(insert var "\t"))
						(delete-char -1))
				(if (= lnum (1- (buffer-lines)))
						(progn
							(dolist (var (butlast *infoman-folder-list* 2))
								(insert var "\t"))
							(insert (nth (1- list-len) *infoman-folder-list*) "\t")
							(insert (nth (- list-len 2) *infoman-folder-list*)))
					(progn
						(dolist (var (butlast *infoman-folder-list* (+ (- list-len lnum) 2)))
							(insert var "\t"))
						(insert (nth (1- lnum) *infoman-folder-list*) "\t")
						(insert (nth (- lnum 2) *infoman-folder-list*) "\t")
						(dolist (var (last *infoman-folder-list* (- list-len lnum)))
							(insert var "\t"))
						(delete-char -1))))
			(setf *infoman-folder-list* (split-string (buffer-substring (point-min) (point-max)) #\TAB))
			(write-file *infoman-folder-list-file-path* t)
			(delete-buffer (selected-buffer))
			(set-buffer (find-buffer *infoman-folder-buffer-name*))
			(infoman-show-folders (1- lnum)))))

(defun infoman-folder-line-down ()
	"ݍs̃tH_ֈړ"
	(interactive)
	(unless (or (eobp)
				(= (current-line-number) (1- (buffer-lines))))
		(let ((lnum (current-line-number))
			  (list-len (list-length *infoman-folder-list*)))
			(infoman-window-reset)
			(set-buffer (create-new-buffer *infoman-work-buffer-name*))
			(if (= lnum 1)
					(progn
						(insert (nth 1 *infoman-folder-list*) "\t")
						(insert (nth 0 *infoman-folder-list*) "\t")
						(dolist (var (last *infoman-folder-list* (- list-len 2)))
							(insert var "\t"))
						(delete-char -1))
				(if (= lnum (- (buffer-lines) 2))
						(progn
							(dolist (var (butlast *infoman-folder-list* 2))
								(insert var "\t"))
							(insert (nth (1- list-len) *infoman-folder-list*) "\t")
							(insert (nth (- list-len 2) *infoman-folder-list*)))
					(progn
						(dolist (var (butlast *infoman-folder-list* (1+ (- list-len lnum))))
							(insert var "\t"))
						(insert (nth lnum *infoman-folder-list*) "\t")
						(insert (nth (1- lnum) *infoman-folder-list*) "\t")
						(dolist (var (last *infoman-folder-list* (1- (- list-len lnum))))
							(insert var "\t"))
						(delete-char -1))))
			(setf *infoman-folder-list* (split-string (buffer-substring (point-min) (point-max)) #\TAB))
			(write-file *infoman-folder-list-file-path* t)
			(delete-buffer (selected-buffer))
			(set-buffer (find-buffer *infoman-folder-buffer-name*))
			(infoman-show-folders (1+ lnum)))))

(defun infoman-rename-folder ()
	"tH_l["
	(interactive)
	(if (and (not (eobp))
			 (if *infoman-message-p*
					 (eq (message-box (format nil
											  "~A l[܂H"
											  (nth (1- (current-line-number)) *infoman-folder-list*))
									  *infoman-frame-name*
									  '(:yes-no :question :button2))
						 :yes)
				 t))
			(let* ((fname-new (read-string "Folder Name: "))
				   (lnum (current-line-number))
				   (list-len (list-length *infoman-folder-list*))
				   (fname-old (nth (1- lnum) *infoman-folder-list*))
				   (folder-path-old (merge-pathnames fname-old *infoman-data-folder-path*)))
				(unless (or (string-equal fname-new "")
							(file-exist-p (merge-pathnames fname-new *infoman-data-folder-path*)))
					(infoman-window-reset)
					(set-buffer (create-new-buffer *infoman-work-buffer-name*))
					(if (= lnum 1)
							(progn
								(insert fname-new)
								(dolist (var (last *infoman-folder-list* (1- list-len)))
									(insert "\t" var)))
						(if (= lnum list-len)
								(progn
									(dolist (var (butlast *infoman-folder-list* 1))
										(insert var "\t"))
									(insert fname-new))
							(progn
								(dolist (var (butlast *infoman-folder-list* (1+ (- list-len lnum))))
									(insert var "\t"))
								(insert fname-new "\t")
								(dolist (var (last *infoman-folder-list* (- list-len lnum)))
									(insert var "\t"))
								(delete-char -1))))
					(setf *infoman-folder-list* (split-string (buffer-substring (point-min) (point-max)) #\TAB))
					(write-file *infoman-folder-list-file-path* t)
					(delete-buffer (selected-buffer))
					(set-buffer (find-buffer *infoman-folder-buffer-name*))
					(rename-file (merge-pathnames fname-old folder-path-old)
								 (merge-pathnames fname-new folder-path-old))
					(rename-file (merge-pathnames (concat fname-old *infoman-data-file-extension*) folder-path-old)
								 (merge-pathnames (concat fname-new *infoman-data-file-extension*) folder-path-old))
					(rename-file (merge-pathnames (concat fname-old ".l") folder-path-old)
								 (merge-pathnames (concat fname-new ".l") folder-path-old))
					(rename-file folder-path-old
								 (merge-pathnames fname-new *infoman-data-folder-path*))
					(infoman-show-folders lnum)))))

(defun infoman-make-folder-index (add-or-del lnum &optional fname)
	"tH_ꗗt@C쐬"
	(let ((list-len (list-length *infoman-folder-list*)))
		(set-buffer (create-new-buffer *infoman-work-buffer-name*))
		(case add-or-del
			(1 (if (= lnum 1)
					   (progn
						   (insert fname "\t")
						   (dolist (var *infoman-folder-list*)
							   (insert var "\t"))
						   (delete-char -1))
				   (if (= lnum (1+ list-len))
						   (progn
							   (dolist (var *infoman-folder-list*)
								   (insert var "\t"))
							   (insert fname))
					   (progn
						   (dolist (var (butlast *infoman-folder-list* (1+ (- list-len lnum))))
							   (insert var "\t"))
						   (insert fname "\t")
						   (dolist (var (last *infoman-folder-list* (1+ (- list-len lnum))))
							   (insert var "\t"))
						   (delete-char -1)))))
			(2 (if (= lnum 1)
					   (progn
						   (dolist (var (last *infoman-folder-list* (1- list-len)))
							   (insert var "\t")))
				   (if (= lnum list-len)
						   (progn
							   (dolist (var (butlast *infoman-folder-list* 1))
								   (insert var "\t")))
					   (progn
						   (dolist (var (butlast *infoman-folder-list* (1+ (- list-len lnum))))
							   (insert var "\t"))
						   (dolist (var (last *infoman-folder-list* (- list-len lnum)))
							   (insert var "\t")))))
			   (delete-char -1)))
		(setf *infoman-folder-list* (split-string (buffer-substring (point-min) (point-max)) #\TAB))
		(write-file *infoman-folder-list-file-path* t)
		(delete-buffer (selected-buffer))
		(set-buffer (find-buffer *infoman-folder-buffer-name*))))

(defun infoman-delete-folder ()
	"Lbgs̃tH_폜"
	(interactive)
	(when (and (not (eobp))
			   (eq (message-box (format nil
										"~A 폜܂H"
										(nth (1- (current-line-number)) *infoman-folder-list*))
								*infoman-frame-name*
								'(:yes-no :exclamation :button2))
				   :yes))
		(let ((dir (directory-namestring (infoman-get-folder-data-path)))
			  (lnum (current-line-number)))
			(infoman-window-reset)
			(directory dir :callback #'(lambda (x)
										   (delete-file (merge-pathnames x dir))))
			(delete-directory dir
							  :if-does-not-exist :error
							  :if-access-denied :error)
			(infoman-make-folder-index 2 lnum)
			(infoman-show-folders lnum))))

(defun infoman-make-data (fname fpath)
	"f[^t@C쐬"
	(let (str)
		(other-window)
		(set-buffer (create-new-buffer *infoman-work-buffer-name*))
		(set-local-window-flags (selected-buffer) *infoman-view-buffer-flags* nil)
		(if (string-equal (setf str (read-string "Header Name: ")) "")
				(insert " ")
			(progn
				(insert (substitute-string str "\t" "  "))
				(while (string-not-equal (setf str (read-string "Header Name: ")) "")
					(insert "\t" (substitute-string str "\t" "  ")))))
		(write-file (merge-pathnames (concat fname *infoman-data-file-extension*) fpath))
		(delete-buffer (selected-buffer))
		(set-buffer (find-buffer *infoman-view-buffer-name*))
		(other-window -1)))

(defun infoman-add-folder ()
	"tH_ǉ"
	(interactive)
	(if (if *infoman-message-p*
				(eq (message-box "tH_ǉ܂H"
								 *infoman-frame-name*
								 '(:yes-no :question :button2))
					:yes)
			t)
			(let* ((fname (read-string "Folder Name: "))
				   (fpath (concat *infoman-data-folder-path* "/" fname))
				   (lnum (current-line-number)))
				(when (and (string-not-equal fname "")
						   (create-directory fpath :if-exists :error))
					(infoman-window-reset)
					(infoman-make-data fname fpath)
					(infoman-make-folder-index 1 lnum fname)
					(infoman-show-folders lnum))
				(toggle-ime nil))))

(defun infoman-folder-mode ()
	"tH_obt@쐬"
	(set-buffer (get-buffer-create *infoman-folder-buffer-name*))
	(kill-all-local-variables)
	(setf mode-name "Folder")
	(setf buffer-mode 'infoman-folder-mode)
	(use-keymap *infoman-folder-mode-map*)
	(make-local-variable 'mode-line-format)
	(setf mode-line-format "%M")
	(make-local-variable 'kept-undo-information)
	(setf kept-undo-information nil)
	(make-local-variable 'need-not-save)
	(setf need-not-save t)
	(make-local-variable 'auto-save)
	(setf auto-save nil)
	(set-buffer-fold-type-none)
	(set-local-window-flags (selected-buffer) *infoman-folder-buffer-flags* nil)
	(setf buffer-read-only t)
	(run-hooks '*infoman-folder-mode-hook*))
