;;; -*- mode: lisp; package: infoman -*-
;;; Last updated: <2005/07/08>

(provide "infoman/grep")

(in-package "infoman")

(defun infoman-grep-close ()
	"Grepobt@"
	(interactive)
	(let ((lnum (current-line-number)))
		(infoman-view-window-reset)
		(delete-buffer *infoman-grep-buffer-name*)
		(set-buffer (find-buffer *infoman-list-buffer-name*))
		(when *infoman-grep-record-line*
			(goto-line (1- (nth (- (list-length *infoman-grep-record-line*)
								   lnum)
								*infoman-grep-record-line*))))))

(defun infoman-grep-open-list ()
	"Lbgs̃XgJ"
	(interactive)
	(unless (eobp)
		(let ((lnum (current-line-number)))
			(infoman-view-window-reset)
			(set-mark-command)
			(set-buffer (find-buffer *infoman-data-buffer-name*))
			(goto-line (nth (- (list-length *infoman-grep-record-line*)
							   lnum)
							*infoman-grep-record-line*))
			(setf *infoman-list-item-value*
				  (split-string (buffer-substring (point)
												  (progn
													  (goto-eol)
													  (point))) #\TAB))
			(goto-bol)
			(set-buffer (find-buffer *infoman-grep-buffer-name*))
			(other-window -1)
			(infoman-view-list))))

(defun infoman-grep-open-list2 ()
	"EBhEړXgJ"
	(interactive)
	(unless (eobp)
		(infoman-view-window-reset)
		(infoman-grep-open-list)
		(other-window)))

(defun infoman-grep-open-previous-list ()
	"EBhEړÕXgJ"
	(interactive)
	(unless (= (current-line-number) 1)
		(infoman-view-window-reset)
		(next-line -1)
		(infoman-grep-open-list)
		(other-window)))

(defun infoman-grep-open-next-list ()
	"EBhEړ̃XgJ"
	(interactive)
	(unless (or (eobp)
				(= (current-line-number) (1- (buffer-lines))))
		(infoman-view-window-reset)
		(next-line)
		(infoman-grep-open-list)
		(other-window)))

(defun infoman-grep-first-error ()
	"XgJ}b`ꏊֈړ"
	(interactive)
	(when (or *infoman-grep-search-string*
			  (string-equal *infoman-grep-search-string* ""))
		(infoman-grep-open-list)
		(scan-buffer *infoman-grep-search-string* :regexp t)
		(start-selection 2 t (match-end 0))))

(defun infoman-grep-mode ()
	"Grepobt@쐬"
	(set-buffer (get-buffer-create *infoman-grep-buffer-name*))
	(kill-all-local-variables)
	(setf mode-name "Grep")
	(setf buffer-mode 'infoman-grep-mode)
	(use-keymap *infoman-grep-mode-map*)
	(make-local-variable 'mode-line-format)
	(setf mode-line-format "%M")
	(make-local-variable 'kept-undo-information)
	(setf kept-undo-information nil)
	(make-local-variable 'need-not-save)
	(setf need-not-save t)
	(make-local-variable 'auto-save)
	(setf auto-save nil)
	(set-local-window-flags (selected-buffer) *infoman-list-buffer-flags* nil)
	(toggle-ime nil)
	(setf *infoman-grep-record-line* nil)
	(run-hooks '*infoman-list-mode-hook*))

(defun infoman-grep-header (&optional header-num)
	"wwb_ڂGrep"
	(interactive)
	(if (find-buffer *infoman-data-buffer-name*)
			(long-operation
				(unless header-num
					(let ((n 0))
						(popup-list *infoman-list-item-header*
									#'(lambda (x)
										  (dolist (var *infoman-list-item-header* header-num)
											  (when (string-equal x var)
												  (setf header-num n))
											  (incf n))))))
				(let ((search-string (completing-read "Search string: "
													  *infoman-completion-list*))
					  record)
					(when (and header-num
							   (string-not-equal search-string ""))
						(infoman-grep-mode)
						(setf *infoman-grep-search-string* search-string)
						(set-buffer (find-buffer *infoman-data-buffer-name*))
						(goto-line 2)
						(with-input-from-buffer ((selected-buffer) (point))
							(goto-line 1)
							(while (next-line)
								(setf record (read-line *standard-input* nil))
								(when (string-matchp search-string (nth header-num (split-string record #\TAB)))
									(push (current-line-number) *infoman-grep-record-line*)
									(set-buffer (find-buffer *infoman-grep-buffer-name*))
									(infoman-insert-list record)
									(set-buffer (find-buffer *infoman-data-buffer-name*)))))
						(set-buffer (find-buffer *infoman-grep-buffer-name*))
						(set-text-attribute (point-min)
											(point-max)
											'grep
											:foreground *infoman-list-fgcolor*
											:bold *infoman-list-bold*)
						(goto-line 1)
						(set-buffer-modified-p nil)
						(setf buffer-read-only t))))))

(defun infoman-grep ()
	"݂̃tH_ŜGrep"
	(interactive)
	(if (find-buffer *infoman-data-buffer-name*)
			(long-operation
				(let ((search-string (completing-read "Search string: "
													  *infoman-completion-list*))
					  record)
					(when (string-not-equal search-string "")
						(infoman-grep-mode)
						(setf *infoman-grep-search-string* search-string)
						(set-buffer (find-buffer *infoman-data-buffer-name*))
						(goto-line 2)
						(while (scan-buffer *infoman-grep-search-string* :regexp t)
							(let (record)
								(setf record (buffer-substring (progn
																   (goto-bol)
																   (point))
															   (progn
																   (goto-eol)
																   (point))))
								(push (current-line-number) *infoman-grep-record-line*)
								(set-buffer (find-buffer *infoman-grep-buffer-name*))
								(infoman-insert-list record)
								(set-buffer (find-buffer *infoman-data-buffer-name*))
								(forward-line)))
						(set-buffer (find-buffer *infoman-grep-buffer-name*))
						(set-text-attribute (point-min)
											(point-max)
											'grep
											:foreground *infoman-list-fgcolor*
											:bold *infoman-list-bold*)
						(goto-line 1)
						(set-buffer-modified-p nil)
						(setf buffer-read-only t))))))
